package com.example.tdl.mqtt;

import com.example.tdl.callabletask.GWConfigWorker;
import com.example.tdl.config.mqtt.MqttConfig;
import com.example.tdl.config.mqtt.MqttTemlateAsync;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.google.gson.JsonSyntaxException;
import org.eclipse.paho.client.mqttv3.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * @Author: lelouch
 * @Description:
 * @Date: Created in 9:41 2018/3/13
 * @Modified By:
 **/
@Component
public class MqttListener implements MqttCallback {
    private static final int INIT_DELAY_DEFAULT = 1000; // unit:ms
    private static final int SCH_PERIOD_DEFAULT = 10 * INIT_DELAY_DEFAULT; // unit:ms
    private static final String CHECKIN_TOPIC = "GW/+/+/CheckIn";

    private MqttTemlateAsync mqttAsyncClient;

    @Autowired
    private MqttConfig mqttconfig;

    @Autowired
    ThreadPoolTaskExecutor configthreadPool;

    Map<String,Future<String>> futureMap = new HashMap<String,Future<String>>();

    private ScheduledExecutorService mReconnectScheduler;
    private long mInitDelay = INIT_DELAY_DEFAULT;
    private long mSchedulePeriod = SCH_PERIOD_DEFAULT;
    private String listener_topic = CHECKIN_TOPIC;

    @PostConstruct
    public void initialize() throws MqttException {
        try {
            mqttAsyncClient = new MqttTemlateAsync(mqttconfig.getUrl() + ":" + mqttconfig.getPort(),
                    "TDL_CheckIn" + "_" + String.valueOf(System.currentTimeMillis()) + "_sub");
                mqttAsyncClient.connect(MqttTemlateAsync.setSSLOptions(mqttconfig.getUsername(), mqttconfig.getPassword(),
                        mqttconfig.getCacrt(), mqttconfig.getClientcrt(),mqttconfig.getClientkey(), mqttconfig.getClientpwd()), null,
                        new IMqttActionListener() {
                            @Override
                            public void onSuccess(IMqttToken iMqttToken) {
                                System.out.println("connection successfull ");
                                try {
                                        mqttAsyncClient.subscribe(listener_topic, mqttconfig.getQos());
                                } catch (MqttException e) {
                                    System.out.println("Error: " + e.getMessage());
                                }
                            }

                            @Override
                            public void onFailure(IMqttToken iMqttToken, Throwable throwable) {
                                System.out.println("connection fail with client id");
                            }
                        });
            this.mqttAsyncClient.setCallback(this);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void clientReconnect() {
        if (mReconnectScheduler != null) {
            // mReconnectScheduler is running.
            return;
        }

        mReconnectScheduler = Executors.newSingleThreadScheduledExecutor();
        mReconnectScheduler.scheduleAtFixedRate(new Runnable() {
            @Override
            public void run() {
                // TODO Auto-generated method stub
                if (mqttAsyncClient != null && !mqttAsyncClient.isConnected()) {
                    try {
                        mqttAsyncClient.reconnect();
                    } catch (MqttSecurityException e) {
                        // TODO: handle exception
                        e.printStackTrace();
                    } catch (MqttException e) {
                        // TODO: handle exception
                        e.printStackTrace();
                    }
                } else {
                    mReconnectScheduler.shutdown();
                    mReconnectScheduler = null;
                }
            }
        }, mInitDelay, mSchedulePeriod, TimeUnit.MILLISECONDS);
    }

    @Override
    public void connectionLost(Throwable cause) {
        System.out.println(cause);
        clientReconnect();
    }

    @Override
    public void messageArrived(String topic, MqttMessage mqttMessage) throws Exception {
        String Message = mqttMessage.toString();
        Long timestamp = System.currentTimeMillis();
        try {
            String[] tmparray = topic.split("/");
            String Type = tmparray[1];
            String SN = tmparray[2];

            String ConfigData ="{\"action\":\"config\"}";

            Future<String> result = null;
            GWConfigWorker gcconfig = new GWConfigWorker(Type,SN,ConfigData, true);
            gcconfig.SetMqttConfig(mqttconfig.getUrl(),mqttconfig.getPort(),
                    mqttconfig.getUsername(), mqttconfig.getPassword(),mqttconfig.getQos(),
                    mqttconfig.getCacrt(),mqttconfig.getClientkey(), mqttconfig.getClientcrt(),mqttconfig.getClientpwd());
            result = configthreadPool.submit(gcconfig);

            while (result.isDone())
            {
                String respcmd = result.get();
            }

   //         futureMap.put(timestamp.toString(),result);


        } catch (JsonSyntaxException e) {
            System.out.println("mqttAsyncClient: message " + Message + " received syntax error.");
        } catch (IllegalStateException e) {
            System.out.println("mqttAsyncClient: " + e);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void deliveryComplete(IMqttDeliveryToken iMqttDeliveryToken) {

    }
}
