package com.example.tdl.web;


import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.GatewayService;
import com.example.tdl.service.ModelService;
import com.example.tdl.service.RoleService;
import com.example.tdl.service.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping("/gateway")
public class GatewayController {

    private Logger logger = LoggerFactory.getLogger(GatewayController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private GatewayService gatewayService;

    @Autowired
    private ModelService modelService;


    //查询所有网关信息
    @ApiOperation(value = "查询所有网关信息",notes = "查询所有网关信息：" +
            "   SN:网关序列号，" +
            "   name:网关名字，" +
            "   type:网关类型，" +
            "   state:网关状态，" +
            "   bConfig:是否配置通道，" +
            "   useScene:使用场景")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(){
        return gatewayService.getAll();
    }

    //添加网关信息
    @ApiOperation(value = "添加网关信息",notes = "添加网关信息，createTime,updateTime不用传，要传的参数：" +
            "   SN:网关序列号，" +
            "   name:网关名字，" +
            "   type:网关类型" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/addGateway",method = RequestMethod.POST)
    public Object addGateway(@RequestBody AddGatewayVo addGatewayVo){
        if (StringUtils.isEmpty(addGatewayVo.getSN())){
            fb.setCode(0);
            fb.setMessage("添加的SN不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addGatewayVo.getName())){
            fb.setCode(0);
            fb.setMessage("添加的名字不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addGatewayVo.getType())){
            fb.setCode(0);
            fb.setMessage("添加的类型不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addGatewayVo.getModelName())){
            fb.setCode(0);
            fb.setMessage("添加的模板名不能为空");
            return gson.toJson(fb);
        }
        if (modelService.getByModelName(addGatewayVo.getModelName())==null){
            fb.setCode(0);
            fb.setMessage("该模板不存在");
            return gson.toJson(fb);
        }
        if (gatewayService.getBySNAndType(addGatewayVo.getSN(),addGatewayVo.getType())!=null){
            fb.setCode(0);
            fb.setMessage("添加的网关已存在");
            return gson.toJson(fb);
        }
        Map<Object,Object> map=new HashMap<>();
        map.put("SN",addGatewayVo.getSN());
        map.put("name",addGatewayVo.getName());
        map.put("type",addGatewayVo.getType());
        map.put("createTime",System.currentTimeMillis());
        map.put("updateTime",System.currentTimeMillis());
        map.put("modelName",addGatewayVo.getModelName());
        gatewayService.addGateway(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("添加网关成功")){
            fb.setCode(1);
            fb.setMessage("添加网关成功");
        }else{
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }
        return gson.toJson(fb);
    }

    //修改网关信息
    @ApiOperation(value = "修改网关信息",notes = "修改网关信息，updateTime不用传，要传的参数：" +
            "   name:网关名字，" +
            "   SN:需要修改的网关序列号，" +
            "   type:需要修改的网关类型")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/updateGateway",method = RequestMethod.POST)
    public Object updateGateway(@RequestBody UpdateGatewayVo updateGatewayVo){
        if (StringUtils.isEmpty(updateGatewayVo.getSN())){
            fb.setCode(0);
            fb.setMessage("修改的SN不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateGatewayVo.getName())){
            fb.setCode(0);
            fb.setMessage("修改的名字不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateGatewayVo.getType())){
            fb.setCode(0);
            fb.setMessage("修改的类型不能为空");
            return gson.toJson(fb);
        }
        if (gatewayService.getBySNAndType(updateGatewayVo.getSN(),updateGatewayVo.getType())==null){
            fb.setCode(0);
            fb.setMessage("修改的设备不存在");
            return gson.toJson(fb);
        }
//        ResultGatewayVo resultGatewayVo=gatewayService.getBySNAndType(updateGatewayVo.getSN(),updateGatewayVo.getType());
//        if (resultGatewayVo!=null&&updateGatewayVo.getSN().equals(updateGatewayVo.getOldSN())){
//            fb.setCode(0);
//            fb.setMessage("修改的设备已存在");
//            return gson.toJson(fb);
//        }
        updateGatewayVo.setUpdateTime(System.currentTimeMillis());
        int a=gatewayService.updateGateway(updateGatewayVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("修改网关成功");
        }else{
            fb.setCode(0);
            fb.setMessage("修改网关失败");
        }
        return gson.toJson(fb);
    }

    //删除网关信息
    @ApiOperation(value = "删除网关信息",notes = "删除网关信息，要传的参数：" +
            "   SN:网关序列号，" +
            "   type:网关类型")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/delGateway",method = RequestMethod.POST)
    public Object delGateway(@RequestBody DelGatewayVo delGatewayVo){
        if (StringUtils.isEmpty(delGatewayVo.getSN())){
            fb.setCode(0);
            fb.setMessage("删除的SN不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(delGatewayVo.getType())){
            fb.setCode(0);
            fb.setMessage("删除的类型不能为空");
            return gson.toJson(fb);
        }
        if (gatewayService.getBySNAndType(delGatewayVo.getSN(),delGatewayVo.getType())==null){
            fb.setCode(0);
            fb.setMessage("删除的网关不存在");
            return gson.toJson(fb);
        }
        Map<Object,Object> map=new HashMap<>();
        map.put("SN",delGatewayVo.getSN());
        map.put("type",delGatewayVo.getType());
        gatewayService.delGateway(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("删除网关成功")){
            fb.setCode(1);
            fb.setMessage("删除网关成功");
        }else{
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }
        return gson.toJson(fb);
    }

    @ApiOperation(value = "使用通道模板配置通道",notes = "使用通道模板配置通道" +
            "       SN：设备编号" +
            "       type：设备类型" +
            "       modelName：模板名")
    @RequestMapping(value = "/deployProbes" ,method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String deployProbes(@RequestBody DeployProbesVo deployProbesVo){
        if(StringUtils.isEmpty(deployProbesVo.getModelName())){
            fb.setCode(0);
            fb.setMessage("模板名不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(deployProbesVo.getSN())){
            fb.setCode(0);
            fb.setMessage("网关序列号不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(deployProbesVo.getType())){
            fb.setCode(0);
            fb.setMessage("网关类型不能为空");
            return gson.toJson(fb);
        }
        if(gatewayService.getBySNAndType(deployProbesVo.getSN(),deployProbesVo.getType())==null){
            fb.setCode(0);
            fb.setMessage("该网关不存在");
            return gson.toJson(fb);
        }
        //根据SN,type获取设备状态
        boolean state = gatewayService.getBySNAndType(deployProbesVo.getSN(),deployProbesVo.getType()).getState();
        if(state){
            fb.setCode(0);
            fb.setMessage("设备在挂载，请先卸载设备");
            return gson.toJson(fb);
        }
        Map<Object,Object> map=new HashMap<>();
        map.put("SN",deployProbesVo.getSN());
        map.put("type",deployProbesVo.getType());
        map.put("modelName",deployProbesVo.getModelName());
        gatewayService.deployGateway(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("通过模板修改设备成功")){
            fb.setCode(1);
            fb.setMessage(msg.get("msg").toString());
        }else{
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }
        return gson.toJson(fb);
    }

}
